@@ -17,7 +17,7 @@
 #'                     \item{V}{Numeric vector. A single explanatory variable (usually a latent variable). Must have the same number of rows as outcomeOrdered.}
 #'                     \item{tau}{Numeric vector. Thresholds. As many as number of different levels in the dependent variable - 1. Extreme thresholds are fixed at -inf and +inf. No mixing allowed in thresholds.}
 #'                     \item{coding}{Numeric or character vector. Optional argument. Defines the order of the levels in \code{outcomeOrdered}. The first value is associated with the lowest level of \code{V}, and the last one with the highest value. If not provided, is assumed to be \code{1:(length(tau) + 1)}.}
-#'                     \item{rows}{Boolean vector. TRUE if a row must be considered in the calculations, FALSE if it must be excluded. It must have length equal to the length of argument \code{choiceVar}. Default value is \code{"all"}, meaning all rows are considered in the calculation.}
+#'                     \item{rows}{Boolean vector. TRUE if a row must be considered in the calculations, FALSE if it must be excluded. It must have length equal to the length of argument \code{outcomeOrdered}. Default value is \code{"all"}, meaning all rows are considered in the calculation.}
 #'                   }
 #' @param functionality Character. Can take different values depending on desired output.
 #'                      \describe{
@@ -48,10 +48,60 @@ apollo_ol  <- function(ol_settings, functionality){
   if(is.null(ol_settings[["rows"]])) ol_settings[["rows"]]="all"
 
   outcomeOrdered=ol_settings[["outcomeOrdered"]]
-  V=ol_settings[["V"]]
-  tau=ol_settings[["tau"]]
-  coding=ol_settings[["coding"]]
-  rows=ol_settings[["rows"]]
+  V      = ol_settings[["V"]]
+  tau    = ol_settings[["tau"]]
+  coding = ol_settings[["coding"]]
+  rows   = ol_settings[["rows"]]
+  nObs <- tryCatch(nrow( get("apollo_inputs", parent.frame(), inherits=FALSE)$database ),
+                   error=function(e){
+                     lenV <- sapply(V, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+                     lenC <- length(outcomeOrdered)
+                     return(max(lenV, lenC))
+                   })
+  
+  ### Format checks
+  # outcomeOrdered
+  test <- is.vector(outcomeOrdered) && (length(outcomeOrdered)==nObs || length(outcomeOrdered)==1)
+  if(!test) stop("The \"outcomeOrdered\" argument needs to be a scalar or a vector with one entry per observation in the \"database\"")
+  # V
+  test <- is.numeric(V) && ( is.vector(V) || (is.array(V) && length(dim(V))<=3) )
+  if(!test) stop("Argument \"V\" must be a numeric vector, matrix or 3-dim array.")
+  test <- (is.vector(V) && length(V) %in% c(1,nObs)) || (is.array(V) && dim(V)[1] %in% c(1,nObs))
+  if(!test) stop("Argument \"V\" must be have 1 row, or as many as the \"database\".")
+  # rows
+  test <- is.vector(rows) && ( (is.logical(rows) && length(rows)==nObs) || (length(rows)==1 && rows=="all") )
+  if(!test) stop("The \"rows\" argument needs to be \"all\" or a vector of boolean statements with one entry per observation in the \"database\"")
+  # functionality
+  test <- functionality %in% c("estimate","prediction","validate","zero_LL","conditionals","output","raw")
+  if(!test) stop("Non-permissable setting for \"functionality\"")
+  # coding
+  test <- is.null(coding) || (is.vector(coding) && !is.null(names(coding)))
+  if(!test) stop("Argument 'rows', if provided, must be \"all\" or a vector of boolean statements of length nObs.")
+  
+  ### Get number of observations
+  nObs <- tryCatch(nrow( get("apollo_inputs", parent.frame(), inherits=TRUE)$database ),
+                   error=function(e){
+                     lenV <- sapply(V, function(v) ifelse(is.array(v), dim(v)[1], length(v)) )
+                     lenC <- length(outcomeOrdered)
+                     return(max(lenV, lenC))
+                   })
+  
+  ### Validate input
+  #if(!is.vector(outcomeOrdered)) stop("Argument 'outcomeOrdered' must be a vector of length nObs.")
+  #if(!is.numeric(V) || !(is.array(V) | is.vector(V))) stop("Argument 'V' must be a numeric vector or array.")
+  #if(!is.null(coding) && (!is.vector(coding) | is.null(names(coding)))) stop("Argument 'coding', if provided, must be a named vector.")
+  ### CORRECTED NEXT LINE
+  ### if(rows!="all" & ( length(rows)!=nObs | !is.logical(rows) )) stop("Argument 'rows', if provided, must be \"all\" or a vector of boolean statements of length nObs.")
+  #if(!( length(rows)==1 && (rows=="all")) & ( length(rows)!=nObs | !is.logical(rows) )) stop("Argument 'rows', if provided, must be \"all\" or a vector of boolean statements of length nObs.")
+  
+  ### Filter rows
+  if(length(rows)==1 && rows=="all") rows <- rep(TRUE, length(nObs))
+  if(any(!rows)){
+    outcomeOrdered <- outcomeOrdered[rows]
+    V <- apollo_keepRows(V, rows)
+  }
+  nObs <- max(ifelse(is.array(V), dim(V)[1], length(V)) , length(outcomeOrdered))
+  
 
   # ############################## #
   #### functionality="validate" ####
@@ -60,28 +110,23 @@ apollo_ol  <- function(ol_settings, functionality){
   if(functionality=="validate"){
     apollo_control <- tryCatch(get("apollo_control", parent.frame(), inherits=FALSE),
                             error = function(e) list(noValidation=FALSE, noDiagnostics=FALSE))
-
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(outcomeOrdered))
-
+    
     if(!apollo_control$noValidation){
       if(!is.vector(tau)) stop("Thresholds for ordered logit needs to be a vector (no random components allowed)!")
-      values_present=unique(outcomeOrdered[rows])
+      values_present=unique(outcomeOrdered)
       if(is.null(coding)){
         coding <- 1:(length(tau)+1)  
-        cat("\nNo coding provided for ordered logit, so assuming outcomeOrdered goes from 1 to",max(coding))
+        cat("\nNo coding provided for ordered logit, so assuming\n outcomeOrdered goes from 1 to",max(coding))
       }
       if(!(all(values_present %in% coding ))) stop("Some levels specified in outcomeOrdered do not exist in coding!")
       if(!(all(coding %in% values_present ))) stop("Some levels specified in coding do not exist in outcomeOrdered!")
-      if( (length(tau)+1)!=length(coding) ) stop("Threshold vector length +1 does not match number of elements in argument 'coding'.")  
-      #cat("\nAll checks passed for Ordered Logit (OL) model component.")
+      if( (length(tau)+1)!=length(coding) ) stop("Threshold vector length +1 does not match number of elements in argument 'coding'.")
     }
-
+    
     if(!apollo_control$noDiagnostics){
-      choicematrix <- t(as.matrix(table(outcomeOrdered[rows])))
-      choicematrix <- rbind(choicematrix, choicematrix[1,]/sum(rows)*100)
+      choicematrix <- t(as.matrix(table(outcomeOrdered)))
+      choicematrix <- rbind(choicematrix, choicematrix[1,]/nObs*100)
       rownames(choicematrix) <- c("Times chosen", "Percentage chosen overall")
-      #cat('Overview of choices for OL model component:\n')
-      #print(round(choicematrix,0))
       apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
       apollo_addLog("Overview of choices for OL model component:", round(choicematrix,0), apolloLog)
     }
@@ -94,10 +139,8 @@ apollo_ol  <- function(ol_settings, functionality){
   # ########################################################## #
 
   if(functionality=="zero_LL"){
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(outcomeOrdered))
-
-    P <- rep(1/(length(tau)+1),length(outcomeOrdered))
-    P[!rows] <- 1
+    P <- rep(NA, nObs)
+    if(any(!rows)) P <- apollo_insertRows(P, rows, 1)
     return(P)
   }
 
@@ -107,11 +150,7 @@ apollo_ol  <- function(ol_settings, functionality){
 
   if(functionality %in% c("estimate","conditionals")){
 
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(outcomeOrdered))
     if(is.null(coding)) coding <- 1:(length(tau)+1)
-    if(functionality=="raw" && length(outcomeOrdered)==1 && is.na(outcomeOrdered)) outcomeOrdered = coding[1]
-    outcomeOrdered[!rows]=coding[1]
-
     map <- stats::setNames(1:length(coding), coding)
     outcomeOrdered2 <- map[as.character(outcomeOrdered)]
 
@@ -119,10 +158,8 @@ apollo_ol  <- function(ol_settings, functionality){
 
     p <- 1/(1 + exp(V-tau[outcomeOrdered2+1])) - 1/(1 + exp(V-tau[outcomeOrdered2]))
 
-    if(is.vector(p)) p[!rows]  <- 1
-    if(is.matrix(p)) p[!rows,] <- 1
-    if(is.array(p) && length(dim(p))==3) p[!rows,,] <- 1
-
+    if(any(!rows)) p <- apollo_insertRows(p, rows, 1) # insert excluded rows with value 1
+    
     return(p)
   }
 
@@ -130,36 +167,23 @@ apollo_ol  <- function(ol_settings, functionality){
   #### functionality="prediction"                           ####
   # ########################################################## #
   if(functionality %in% c("prediction", "raw")){
-
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(outcomeOrdered))
-    outcomeOrdered[!rows] <- 1
+    
+    if(is.null(coding)) coding <- 1:(length(tau)+1)
     tau <- c(-Inf,tau,Inf)
-
+    
     p = list()
-    j=1
-    while(j<= length(tau)-1){
-      p[[j]] = 1/(1 + exp(V-tau[j+1])) - 1/(1 + exp(V-tau[j]))
-      j = j+1
-    }
-
+    for(j in 1:(length(tau)-1)) p[[j]] = 1/(1 + exp(V-tau[j+1])) - 1/(1 + exp(V-tau[j]))
+    
     if(is.null(coding)) coding <- 1:(length(tau)-1)
     names(p) <- coding
-	
-	# Add chosen alternative (unless outcomeOrdered is NA)
+    
+    # insert excluded rows with value NA
+	  if(any(!rows)) p <- lapply(p, apollo_insertRows, r=rows, val=NA)
+    
+    # Add chosen alternative (unless outcomeOrdered is NA)
     if(!(length(outcomeOrdered)==1 && is.na(outcomeOrdered))){
-      map <- stats::setNames(1:length(coding), coding)
-      outcomeOrdered2  <- map[as.character(outcomeOrdered)]
-      p[["chosen"]] <- 1/(1 + exp(V-tau[outcomeOrdered2+1])) - 1/(1 + exp(V-tau[outcomeOrdered2]))
+      p[["chosen"]] <- apollo_ol(ol_settings, functionality="estimate")
     }
-	
-	# change likelihood of excluded columns for NA (prediction) or 1 (raw)
-    p <- lapply(p, function(x) {
-      tmp <- ifelse(functionality=="prediction",NA,1)
-      if(is.vector(x)) x[!rows]  <- tmp
-      if(is.matrix(x)) x[!rows,] <- tmp
-      if(is.array(x) & length(dim(x))==3) x[!rows,,] <- tmp
-      return(x)
-    })
 
     return(p)
   }
@@ -170,35 +194,10 @@ apollo_ol  <- function(ol_settings, functionality){
   if(functionality=="output"){
 
     p <- apollo_ol(ol_settings, functionality="estimate")
-
-    if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(outcomeOrdered))
-    choicematrix <- t(as.matrix(table(outcomeOrdered[rows])))
-    choicematrix <- rbind(choicematrix, choicematrix[1,]/sum(rows)*100)
+    
+    choicematrix <- t(as.matrix(table(outcomeOrdered)))
+    choicematrix <- rbind(choicematrix, choicematrix[1,]/nObs*100)
     rownames(choicematrix) <- c("Times chosen", "Percentage chosen overall")
-
-    ## write diagnostics to a file named "modelName_tempOutput.txt" in a temporary directory.
-    #apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=FALSE )$apollo_control,
-    #                            error=function(e){
-    #                              cat("apollo_ol could not retrieve apollo_control. No diagnostics in output.\n")
-    #                              return(NA)
-    #                            } )
-    #if(!(length(apollo_control)==1 && is.na(apollo_control))){
-    #  fileName <- paste(apollo_control$modelName, "_tempOutput.txt", sep="")
-    #  fileName <- file.path(tempdir(),fileName)
-    #  fileConn <- tryCatch( file(fileName, open="at"),
-    #                        error=function(e){
-    #                          cat('apollo_ol could not write diagnostics to temporary file. No diagnostics in output.\n')
-    #                          return(NA)
-    #                        })
-    #  if(!anyNA(fileConn)){
-    #    sink(fileConn)
-    #    on.exit({if(sink.number()>0) sink(); close(fileConn)})
-    #    if(apollo_control$noDiagnostics==FALSE){
-    #      cat('Overview of choices for OL model component:\n')
-    #      print(round(choicematrix,0))
-    #      cat('\n')}
-    #  }
-    #}
     
     apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
     apollo_addLog("Overview of choices for OL model component:", round(choicematrix,0), apolloLog)
